# SPDX-FileCopyrightText: Copyright (c) 2023-2025, Kr8s Developers (See LICENSE for list)
# SPDX-License-Identifier: BSD 3-Clause License
import json
import sys

import typer
import yaml
from rich.console import Console
from rich.syntax import Syntax

import kr8s

console = Console()


async def version(
    client: bool = typer.Option(
        False,
        "--client",
        help="If true, shows client version only (no server required).",
    ),
    output: str = typer.Option(
        "",
        "-o",
        "--output",
        help="One of 'yaml' or 'json'.",
    ),
):
    """Print the client and server version information for the current context.

    Examples:
        # Print the client and server versions for the current context
        kubectl version
    """
    versions = {}
    versions["clientVersion"] = {
        "client": "kubectl-ng",
        "gitVersion": kr8s.__version__,
        "major": kr8s.__version__.split(".")[0],
        "minor": kr8s.__version__.split(".")[1],
        "pythonVersion": sys.version,
    }
    if not client:
        api = await kr8s.asyncio.api()
        versions["serverVersion"] = await api.version()

    if output == "":
        style = "[magenta][bold]"
        client_version = versions["clientVersion"]["gitVersion"]
        console.print(f"Client Version: {style}v{client_version}")
        server_version = versions["serverVersion"]["gitVersion"]
        console.print(f"Server Version: {style}{server_version}")

    elif output == "yaml":
        console.print(
            Syntax(
                yaml.dump(versions),
                "yaml",
                background_color="default",
            )
        )

    elif output == "json":
        console.print(
            Syntax(
                json.dumps(versions, indent=2),
                "json",
                background_color="default",
            )
        )

    else:
        console.print("error: --output must be 'yaml' or 'json'")
        raise typer.Exit(code=1)



# SPDX-FileCopyrightText: Copyright (c) 2023-2025, Kr8s Developers (See LICENSE for list)
# SPDX-License-Identifier: BSD 3-Clause License

import anyio
import typer
from rich.console import Console

import kr8s.asyncio
from kr8s.asyncio.objects import objects_from_files

console = Console()


# Missing Options
# TODO --all=false
# TODO -A, --all-namespaces=false
# TODO --cascade='background'
# TODO --dry-run='none'
# TODO --field-selector=''
# TODO --force=false
# TODO --grace-period=-1
# TODO --ignore-not-found=false
# TODO -k, --kustomize=''
# TODO --now='false'
# TODO -o, --output=''
# TODO --raw='false'
# TODO -R, --recursive='false'
# TODO -l, --selector=''
# TODO --timeout='0s'
# TODO --wait='true'


async def delete(
    filename: str = typer.Option(
        "",
        "--filename",
        "-f",
        help="Filename, directory, or URL to files identifying the resources to delete",
    ),
    wait: bool = typer.Option(
        True,
        "--wait",
        help="If true, wait for resources to be gone before returning. This waits for finalizers.",
    ),
):
    api = await kr8s.asyncio.api()
    try:
        objs = await objects_from_files(filename, api)
    except Exception as e:
        console.print(f"[red]Error loading objects from {filename}[/red]: {e}")
        raise typer.Exit(1)
    for obj in objs:
        try:
            await obj.delete()
        except Exception as e:
            console.print(f"[red]Error deleting {obj}[/red]: {e}")
            raise typer.Exit(1)
        console.print(f'[green]{obj.singular} "{obj}" deleted [/green]')
    async with anyio.create_task_group() as tg:
        for obj in objs:
            if wait:
                tg.start_soon(obj.wait, "delete")



# SPDX-FileCopyrightText: Copyright (c) 2024-2025, Kr8s Developers (See LICENSE for list)
# SPDX-License-Identifier: BSD 3-Clause License
import json

import yaml
from kubectl_ng.cli import app
from typer.testing import CliRunner

runner = CliRunner()


def test_version():
    result = runner.invoke(app, ["version"])
    assert result.exit_code == 0
    assert "Client Version" in result.stdout
    assert "Server Version" in result.stdout


def test_version_yaml():
    result = runner.invoke(app, ["version", "-o", "yaml"])
    assert result.exit_code == 0
    data = yaml.safe_load(result.stdout)
    assert "clientVersion" in data


def test_version_json():
    result = runner.invoke(app, ["version", "-o", "json"])
    assert result.exit_code == 0
    data = json.loads(result.stdout)
    assert "clientVersion" in data


def test_version_invalid_output():
    result = runner.invoke(app, ["version", "-o", "foo"])
    assert result.exit_code == 1



# SPDX-FileCopyrightText: Copyright (c) 2023-2025, Kr8s Developers (See LICENSE for list)
# SPDX-License-Identifier: BSD 3-Clause License
import typer
from rich import box
from rich.console import Console
from rich.table import Table

import kr8s

console = Console()


async def api_resources(
    output: str = typer.Option(
        None,
        "-o",
        "--output",
        help="Output format. One of: (wide, name).",
    ),
    api_group: str = typer.Option(
        "", "--api-group", help="Limit to resources in the specified API group."
    ),
    categories: str = typer.Option(
        "",
        "--categories",
        help="Limit to resources that belong the the specified categories. Comma separated list of categories.",
    ),
    namespaced: str = typer.Option(
        None,
        "--namespaced",
        help="If false, non-namespaced resources will be returned, "
        "otherwise returning namespaced resources by default.",
    ),
    headers: bool = typer.Option(
        True,
        help="When using the default or custom-column output format, don't print headers (default print headers).",
    ),
    sort_by: str = typer.Option(
        None,
        "--sort-by",
        help="If non-empty, sort list of resources using specified field. The field can be either 'name' or 'kind'.",
    ),
    verbs: str = typer.Option(
        "",
        "--verbs",
        help="Limit to resources that support the specified verbs. Comma separated list of verbs.",
    ),
):
    """Print the supported API resources on the server.

    Examples:
        # Print the supported API resources
        kubectl-ng api-resources

        # Print the supported API resources with more information
        kubectl-ng api-resources -o wide

        # Print the supported API resources sorted by a column
        kubectl-ng api-resources --sort-by=name

        # Print the supported namespaced resources
        kubectl-ng api-resources --namespaced=true

        # Print the supported non-namespaced resources
        kubectl-ng api-resources --namespaced=false

        # Print the supported API resources with a specific APIGroup
        kubectl-ng api-resources --api-group=rbac.authorization.k8s.io

    """
    kubernetes = await kr8s.asyncio.api()
    categories = [c.strip() for c in categories.split(",")] if categories else None
    verbs = [v.strip() for v in verbs.split(",")] if verbs else None

    resources = await kubernetes.api_resources()

    table = Table(box=box.SIMPLE, show_header=headers)
    table.add_column("Name", style="cyan", no_wrap=True)
    table.add_column("Short Names")
    table.add_column("API Version")
    table.add_column("Namespaced")
    table.add_column("Kind")
    if output == "wide":
        table.add_column("Verbs", no_wrap=True)
        table.add_column("Categories", no_wrap=True)

    if output == "name":
        for resource in resources:
            if resource["version"] != "v1":
                console.print(f"{resource['name']}.{resource['version'].split('/')[0]}")
            else:
                console.print(resource["name"])
        return

    if sort_by == "name":
        resources = sorted(resources, key=lambda r: r["name"])
    elif sort_by == "kind":
        resources = sorted(resources, key=lambda r: r["kind"])

    for resource in resources:
        if (
            (not api_group or resource["version"].startswith(api_group))
            and (
                categories is None
                or set(categories).issubset(set(resource.get("categories", [])))
            )
            and (
                namespaced is None or str(resource["namespaced"]).lower() == namespaced
            )
            and (verbs is None or set(verbs).issubset(set(resource.get("verbs", []))))
        ):
            if output == "name":
                if resource["version"] != "v1":
                    console.print(
                        f"{resource['name']}.{resource['version'].split('/')[0]}"
                    )
                else:
                    console.print(resource["name"])
            data = [
                resource["name"],
                "[magenta]" + ",".join(resource.get("shortNames", [])),
                resource["version"],
                "[green]true" if resource["namespaced"] else "[indian_red1]false",
                resource["kind"],
            ]
            if output == "wide":
                data.append(",".join(resource["verbs"]))
                data.append(",".join(resource.get("categories", [])))
            table.add_row(*data)

    if output != "name":
        console.print(table)
